module PatriotHadoop
  module Ext
    module Hive

      HIVE_MAX_ERROR_MSG_SIZE = 512

      include Patriot::Util::Logger
      include Patriot::Util::DBClient
      include Patriot::Util::System

      def self.included(cls)
        cls.send(:include, Patriot::Util::System)
      end

      class HiveException < Exception; end


      def execute_hivequery(hql_file, output_file=nil, user=nil)
        command = "hive -f \"#{hql_file}\"#{_compress_option(File.extname(output_file))}"
        unless user.nil?
          if user !~ /^[a-z_][a-z0-9_]{0,30}$/
            raise HiveException, "Invalid username" 
          end
          command = "sudo -u #{user} #{command}"
        end
        return _execute_hivequery_internal(command, output_file)
      end


      def _compress_option(extension)
        return case extension
          when '.gz'  then ' | gzip --stdout --force'
          when '.bz2' then ' | bzip2 --stdout --force'
          else ''
        end
      end


      def _execute_hivequery_internal(command, output_file)
        res = execute_command(command) do |status, so, se|
          err_size = File.stat(se).size
          err_msg  = ""
          max_err_size = HIVE_MAX_ERROR_MSG_SIZE
          File.open(se) do |f|
            if err_size > max_err_size
              f.seek(-1 * max_err_size, IO::SEEK_END)
              err_msg = "\n(#{err_size - max_err_size} bytes are truncated)"
            end
            err_msg = "#{f.read}#{err_msg}"
          end
          raise HiveException, "#{command}\n#{err_msg}"
        end
        File.rename(res, output_file) unless output_file.nil?
      end

    end
  end
end
